//
//  EMLocationViewController.m
//  ChatDemo-UI3.0
//
//  Created by XieYajie on 2019/1/29.
//  Copyright © 2019 XieYajie. All rights reserved.
//

#import "EMLocationViewController.h"
#import "EaseHeaders.h"
#import "EaseAlertController.h"
#import "EaseAlertView.h"

@interface EMLocationViewController ()<MKMapViewDelegate, CLLocationManagerDelegate>

@property (nonatomic) BOOL canSend;

@property (nonatomic) CLLocationCoordinate2D locationCoordinate;
@property (nonatomic, strong) NSString *address;
@property (nonatomic, strong) NSString *buildingName;

@property (nonatomic, strong) MKMapView *mapView;
@property (nonatomic, strong) MKPointAnnotation *annotation;
@property (nonatomic, strong) CLLocationManager *locationManager;

@property (nonatomic, strong) UIButton *sendLocationBtn;
@property (nonatomic, strong) UIButton *cancelBtn;

@end

@implementation EMLocationViewController

- (instancetype)init
{
    self = [super init];
    if (self) {
        _canSend = YES;
    }
    
    return self;
}

- (instancetype)initWithLocation:(CLLocationCoordinate2D)aLocationCoordinate
{
    self = [super init];
    if (self) {
        _canSend = NO;
        _locationCoordinate = aLocationCoordinate;
    }
    
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [self _setupSubviews];
    
    if (self.canSend) {
        self.mapView.showsUserLocation = YES;//显示当前位置
        [self _startLocation];
    } else {
        [self _moveToLocation:self.locationCoordinate];
    }
}

#pragma mark - Subviews

- (void)_setupSubviews
{
    /*
    [UIApplication sharedApplication].statusBarStyle = UIStatusBarStyleDefault;
    self.navigationController.navigationBar.barStyle = UIBarStyleDefault;
    [self.navigationController.navigationBar setBackgroundImage:[UIImage easeUIImageNamed:@"navbar_white"] forBarMetrics:UIBarMetricsDefault];
    [self.navigationController.navigationBar.layer setMasksToBounds:YES];
    
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithImage:[[UIImage easeUIImageNamed:@"close_gray"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal] style:UIBarButtonItemStylePlain target:self action:@selector(closeAction)];
    if (self.canSend) {
        self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:EaseLocalizableString(@"send", nil) style:UIBarButtonItemStylePlain target:self action:@selector(sendAction)];
    }
    self.title = @"地理位置";*/
    self.navigationController.navigationBar.hidden = YES;
    self.view.backgroundColor = [UIColor whiteColor];
    self.mapView = [[MKMapView alloc] initWithFrame:self.view.bounds];
    self.mapView.delegate = self;
    self.mapView.mapType = MKMapTypeStandard;
    self.mapView.zoomEnabled = YES;
    [self.view addSubview:self.mapView];
    
    [self.view addSubview:self.sendLocationBtn];
    [self.sendLocationBtn Ease_makeConstraints:^(EaseConstraintMaker *make) {
        make.width.equalTo(@55);
        make.height.equalTo(@25);
        make.top.equalTo(self.view).offset(55);
        make.right.equalTo(self.view).offset(-24);
    }];
    if (!self.canSend) {
        self.sendLocationBtn.hidden = YES;
    }
    
    [self.view addSubview:self.cancelBtn];
    [self.cancelBtn Ease_makeConstraints:^(EaseConstraintMaker *make) {
        make.width.equalTo(@55);
        make.height.equalTo(@25);
        make.top.equalTo(self.view).offset(55);
        make.left.equalTo(self.view).offset(24);
    }];
    
    self.annotation = [[MKPointAnnotation alloc] init];
}

#pragma mark - Private

- (void)_startLocation
{
    if ([CLLocationManager locationServicesEnabled]) {
        _locationManager = [[CLLocationManager alloc] init];
        _locationManager.delegate = self;
        _locationManager.distanceFilter = 5;
        _locationManager.desiredAccuracy = kCLLocationAccuracyBestForNavigation;//kCLLocationAccuracyBest;
        if ([[UIDevice currentDevice].systemVersion floatValue] >= 8.0) {
            [_locationManager requestWhenInUseAuthorization];
        }
    }
    
    [self showHudInView:self.view hint:EaseLocalizableString(@"Locating...", nil)];
}

- (void)_moveToLocation:(CLLocationCoordinate2D)locationCoordinate
{
    [self hideHud];
    
    self.locationCoordinate = locationCoordinate;
    float zoomLevel = 0.01;
    MKCoordinateRegion region = MKCoordinateRegionMake(self.locationCoordinate, MKCoordinateSpanMake(zoomLevel, zoomLevel));
    [self.mapView setRegion:[self.mapView regionThatFits:region] animated:YES];
    
    [self.mapView removeAnnotation:self.annotation];
    self.annotation.coordinate = locationCoordinate;
    [self.mapView addAnnotation:self.annotation];
}

#pragma mark - MKMapViewDelegate

- (void)mapView:(MKMapView *)mapView didUpdateUserLocation:(MKUserLocation *)userLocation
{
    __weak typeof(self) weakself = self;
    CLGeocoder *geocoder = [[CLGeocoder alloc] init];
    [geocoder reverseGeocodeLocation:userLocation.location completionHandler:^(NSArray *array, NSError *error) {
        if (!error && array.count > 0) {
            CLPlacemark *placemark = [array objectAtIndex:0];
            weakself.address = placemark.name;
            weakself.buildingName = @"建筑物名称";//自主获取
            [weakself _moveToLocation:userLocation.coordinate];
        }
    }];
}

- (void)mapView:(MKMapView *)mapView didFailToLocateUserWithError:(NSError *)error
{
    [self hideHud];
    if (error.code == 0) {
        EaseAlertView *alertView = [[EaseAlertView alloc]initWithTitle:nil message:[error.userInfo objectForKey:NSLocalizedRecoverySuggestionErrorKey]];
        [alertView show];
    }
}

- (void)locationManager:(CLLocationManager *)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status
{
    switch (status) {
        case kCLAuthorizationStatusNotDetermined:
            if ([self.locationManager respondsToSelector:@selector(requestAlwaysAuthorization)]) {
                [self.locationManager requestWhenInUseAuthorization];
            }
            break;
        case kCLAuthorizationStatusDenied:
            break;
        default:
            break;
    }
}

#pragma mark - Action

- (void)closeAction
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)sendAction
{
    [self dismissViewControllerAnimated:YES completion:^{
        if (self.sendCompletion) {
            self.sendCompletion(self.locationCoordinate, self.address, self.buildingName?self.buildingName:@"");
        }
    }];
}

#pragma mark - Getter

- (UIButton *)sendLocationBtn
{
    if (_sendLocationBtn == nil) {
        _sendLocationBtn = [[UIButton alloc]init];
        [_sendLocationBtn setTitle:EaseLocalizableString(@"send", nil) forState:UIControlStateNormal];
        _sendLocationBtn.layer.cornerRadius = 4;
        _sendLocationBtn.backgroundColor = [UIColor colorWithRed:72/255.0 green:200/255.0 blue:144/255.0 alpha:1.0];
        [_sendLocationBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_sendLocationBtn.titleLabel setFont:[UIFont systemFontOfSize:12.0]];
        [_sendLocationBtn addTarget:self action:@selector(sendAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _sendLocationBtn;
}

- (UIButton *)cancelBtn
{
    if (_cancelBtn == nil) {
        _cancelBtn = [[UIButton alloc]init];
        [_cancelBtn setTitle:EaseLocalizableString(@"cancel", nil) forState:UIControlStateNormal];
        _cancelBtn.backgroundColor = [UIColor clearColor];
        [_cancelBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [_cancelBtn.titleLabel setFont:[UIFont systemFontOfSize:14.0]];
        [_cancelBtn addTarget:self action:@selector(closeAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _cancelBtn;
}

@end
